/* Function File for ProTrinket Mouse Program v2
 * by Sinclair Gurny
 * Aug 17
 */

const int def_val = HIGH;

// ===== Mouse functions =====

// Reads an axis of the joystick and parses the value
//   input value [0, 1023] to [0, range]
int readJoyAxis( int axis ) {
  int reading = analogRead( axis );
  // map value to output range
  reading = map( reading, 0, 1023, 0, range );
  // center output reading
  int dist = center - reading;
  // to prevent jitters
  if ( abs(dist) < threshold ) {
    dist = 0;
  }
  return dist;
}

// Reads middle click value
int midClick() {
#if MIDCL_MODE
  return digitalRead( MID_CLICK );
#else
  return def_val;
#endif
}

// Reads page mode value
int pageBtn() {
#if PG_MODE
  return digitalRead( PAGE_MODE );
#else
  return def_val;
#endif
}

// Reads scroll mode value
int scrollBtn() {
#if SCRL_MODE
  return digitalRead( SCROLL_MODE );
#else
  return def_val;
#endif
}


// ====== Gyro functions =====
// Takes acceleration values (milli-g) and converts into
//   orientation
// FORWARD - 0
// RIGHT   - 1
// BACK    - 2
// LEFT    - 3
// else    - -1
int orientation( int accX, int accY ) {
  if ( accX < -700 && (accY > -200 && accY < 200)) {
    // LEFT
    return 3;
  } else if ( accX > 700 && (accY > -200 && accY < 200)) {
    // RIGHT
    return 1;
  } else if ( (accX > -200 && accX < 200) && accY < -700 ) {
    // FORWARD
    return 0;
  } else if ( (accX > -200 && accX < 200) && accY > 700 ) {
    // BACK
    return 2;
  }
  return -1;
}

// ======= BTNS =========

// Adds a modifier or normal key into the key lists
void Key_Pressed( uint8_t mkey, uint8_t nkey ) {
  int has_changed = 0; // keeps track if a new key has been pressed
  // === Add modifier key ===
  uint8_t before = mod_key;
  mod_key |= mkey;
  if ( mod_key != before ) {
    has_changed |= 1;
  }

  // === Add other keys ===
  // look for if newkey is already listed
  int i = 0;
  for ( ; i < keys_idx; ++i ) {
    if ( keys[ i ] == nkey ) {
      break;
    }
  }
  if ( i == keys_idx && keys_idx < 10 ) {
    // key not pressed, add to list
    has_changed |= 2; 
    keys[ keys_idx ] = nkey;
    ++keys_idx;
  }
  // if any new key was added then increase count
  if ( has_changed > 0 ) {
    ++keys_pressed;
  }
}

// ======== Haptic ==========

void vibrate( int pattern ) {
#if HAPTIC_MODE
  drv.setWaveform( 0, pattern );
  drv.setWaveform( 1, 0 );
  drv.go();
#endif
}



